# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from command import command, swrapper
from classifier import classifier
from cxutil import flags,utils,ip
from cxutil import pmask,cmask
from cxutil.exceptions import Dump
from cxutil.UID import UID

import os
import re
import types

class network (command):
	"""
	Configure network parameters
	"""
	mask = flags.Begin

	export = [
		"egress",
		("simple-acl","simple_acl"),
	]

	line = re.compile("^[0-9]+.*$")

	###
	# create default children
	###
	children = ["l7"]

	###
	# iptables chain map
	###
	chainmap = {
		"in": {
			"key": "-i",
			"accept": ("filter","INPUT"),
			"reject": ("filter","INPUT"),
			"drop": ("filter","INPUT"),
			"dnat": ("nat","PREROUTING"),
			"snat": ("nat","POSTROUTING"),
			"set": ("mangle","PREROUTING"),
			"masquerade": ("nat","POSTROUTING"),
		},
		"out": {
			"key": "-o",
			"accept": ("filter","OUTPUT"),
			"reject": ("filter","OUTPUT"),
			"drop": ("filter","OUTPUT"),
			"dnat": ("nat","OUTPUT"),
			"snat": ("nat","POSTROUTING"),
			"set": ("mangle","POSTROUTING"),
			"masquerade": ("nat","POSTROUTING"),
		},
		"forward": {
			"key": "-i",
			"accept": ("filter","FORWARD"),
			"reject": ("filter","FORWARD"),
			"drop": ("filter","FORWARD"),
			"dnat": ("nat","PREROUTING"),
			"snat": ("nat","POSTROUTING"),
			"set": ("mangle","FORWARD"),
			"masquerade": ("nat","POSTROUTING"),
		}
	}


	def acl_compare(self,t1,t2):

		# return 0 if lines are equal
		if t1 == t2:
			return 0
		
		# standard targets must be ignored
		t1 = t1.split("-")
		t2 = t2.split("-")
		if len(t1) != 4:
			return None

		# ignore the first word?
		
		# sort by the second word: system < .*
		if (t2[1] == "system") and (t1[1] != "system"):
			# t1 > t2
			return 1
		if (t2[1] != "system") and (t1[1] == "system"):
			# t1 < t2
			return -1

		# sort by the third word: in < forward < out
		if t2[2] < t1[2]:
			# t1 > t2
			return 1
		if t2[2] > t1[2]:
			# t1 < t2
			return -1

		# sort by priority:
		if t2[3] < t1[3]:
			# t1 > t2
			return 1
		if t2[3] > t1[3]:
			# t1 < t2
			return -1

		return 1


	def assemble_line(self,assembled,key,target):
		k = 0
		for i in assembled:
			x = self.acl_compare(i.target,target)
			if type(x) == types.NoneType:
				# ignore the line,
				continue
			elif x == 0:
				# targets are equal:
				i.flush = True
				return
			elif x > 0:
				# assembled > new target,
				assembled.insert(k,utils.opts({"processed": False, "flush": False, "target": target, "key": key}))
				return
			k += 1

		assembled.append(utils.opts({"processed": False, "flush": False, "target": target, "key": key}))
		return
			

	def assemble_from_iptables(self,lines):
		result = []
		for i in lines:
			if self.line.match(i):
				i = i.split()
				result.append(utils.opts({
					"processed": True,
					"flush": False,
					"key": "",
					"target": i[3],
				}))
		return result

	def postUp(self,opts):
		###
		# We can not address a rule in iptables by an ID: there is no such ID.
		# So, we can not insert a rule in a certain position, we can only INSERT
		# it into ANY (not certain) position or APPEND it to the end of a chain.
		# Because of this brainf@ck, we can not guarantee the rule scan order after
		# a deletion AND an insertion of the changed rule.
		#
		# There are two ways to overcome this grief:
		#	1. Flush ALL tables, delete ALL user-defined chains and reinstall
		#	  ALL rules from scratch.
		#	2. List a table and try to calculate a rule number
		#
		# The second way is quite hard to go. Very hard. Too hard :(((
		###

		s = swrapper()
		t = swrapper()
		# s.add_command("message",opts.rest_fc.dump_recursive())

		##
		# ACL UUID structure:
		#	f1-f2-f3
		#	* f1: "system" or interface name, e.g. "eth0" and so on
		#	* f2: direction: "in", "forward", "out"
		#	* f3: ACL priority
		#
		if not opts.rest_fc.has_key("acls"):
			return s

		acls = opts.rest_fc.acls
		asserts = {}
		assembled = {}

		for i in acls.keys():
			###
			# First word of ACL UUID: i (the key)
			###
			interface = i
			for k in acls[i].keys():
				###
				# Second word of ACL UUID: k (direction)
				###
				direction = k
				for l in acls[i][k].keys():
					###
					# Third word of ACL UUID: l (priority)
					###
					uuid = "%s-%s-%s" % (i,k,l)
					rules = acls[i][k][l].keys()
					rules.sort(utils.nsort)


					for m in rules:
						n = acls[i][k][l][m]

						if interface:
						    key = "%s %s" % (self.chainmap[direction]["key"],interface)
						else:
						    key = ""
						    
						rulenum = n["$1"]
						action = n["$2"]

						(table,chain) = self.chainmap[direction][action]
						schain = "%s-%s" % (chain,uuid)

						if not asserts.has_key(table):
							asserts[table] = []
						if not schain in asserts[table]:
							asserts[table].append(schain)
							# assert interface chain
							if not assembled.has_key(table):
								assembled[table] = utils.opts()
							if not assembled[table].has_key(chain):
								x = utils.Executor("iptables -n -v --line-numbers -t %s -L %s" % (table,chain))
								assembled[table][chain] = self.assemble_from_iptables(x.lines)
							self.assemble_line(assembled[table][chain],key,schain)

						o = ""

						# general switches
						if n.has_key("src"):
							o += " -s %s" % (n["src"])
						if n.has_key("dst"):
							o += " -d %s" % (n["dst"])
						if n.has_key("proto"):
							o += " -p %s" % (n["proto"])

						# modules
						if n.has_key("fwmark") and (action != "set"):
							o += " -m mark --mark %s" % (n["fwmark"])
						if n.has_key("dport") and n.has_key("proto"):
							o += " -m %s --destination-port %s" % (n["proto"],n["dport"])
						if n.has_key("sport") and n.has_key("proto"):
							o += " -m %s --source-port %s" % (n["proto"],n["sport"])

						# actions
						if action == "set":
							o += " -j MARK --set-mark %s" % (n["fwmark"])
						else:
							o += " -j %s" % (action.upper())
							if n.has_key("tosrc") and (action == "snat"):
								o += " --to-source %s" % (n["tosrc"])
							if n.has_key("todst") and (action == "dnat"):
								o += " --to-destination %s" % (n["todst"])

						# attach a rule
						s.add_commands("exec",[
							"iptables -t %s -A %s %s" % (table,schain,o),
						])

		###
		# i: table
		# l: chain
		# n: schain reference
		for (i,k) in assembled.items():
			for (l,m) in k.items():
				o = 1
				for n in m:
					if not n.processed:
						t.add_commands("exec",[
							"iptables -t %s -N %s" % (i,n.target),
							"iptables -t %s -I %s %s %s -j %s" % (i,l,o,n.key,n.target),
						])
					if n.flush:
						t.add_commands("exec",[
							"iptables -t %s -F %s" %(i,n.target),
						])
					o += 1
		t.merge([s,])
		return t

## 8<---------------------------------------------------------------------------------

	class simple_acl (command):
		"""
		Configure ACL rules
		"""
		mask = flags.Begin
		name = "simple-acl"

		parameters = {
			"$1": {
				"description": "ACL name (any string)",
				"name": "acl",
				"type": "string",
			}
		}
		modules = [
			"rule",
		]


		def postUp(self,opts):

			###
			# pack restack and send it to a parent node
			###

			direction = "in"
			priority = "9000"

			if opts.envl.has_key("fps"):
				# we're called as a function
				# get our direction and priority
				fps = opts.envl.fps.split()
				direction = fps[0]
				if len(fps) == 2:
					priority = fps[1]
				if len(fps) == 3:
					priority = fps[2]


			if opts.envl.has_key("interface"):
				# we're called from an interface
				# get the interface id
				interface = opts.envl.interface
			else:
				return

			# create ACL pack
			opts.rest_tp.acls = {}
			opts.rest_tp.acls[interface] = {}
			opts.rest_tp.acls[interface][direction] = {}
			opts.rest_tp.acls[interface][direction][priority] = opts.envl.matches

			return 

				

		class rule (classifier):
			"""
			Traffic bypass rule
			"""
			parameters = {
				"$1": {
					"description": "Rule priority",
					"type": "int",
					"name": "priority",
				},

				"$2": {
					"description": "Rule action",
					"type": "enum",
					"name": "action",
					"range": ("accept","reject","drop","dnat","snat","masquerade","set"),
				},

				"tosrc": {
					"description": "Source IP for SNAT",
					"type": "ipaddr",
					"mask": pmask.Optional,
				},

				"todst": {
					"description": "Destination IP for DNAT",
					"type": "ipaddr",
					"mask": pmask.Optional,
				},
			}

			def __init__(self):
				command.__init__(self)
				utils.merge(classifier.parameters,self.parameters)

		        def pre(self,opts):
				command.pre(self,opts)
				if not opts.envl.has_key("matches"):
					opts.envl.matches = {}

				opts.envl.matches[opts.envl.priority] = opts.opts.parameters



## 8<---------------------------------------------------------------------------------

	class egress (command):
		"""
		Configure egress queue
		"""
		mask = flags.Begin | flags.Force

		parameters = {
			"$1": {
				"type": "enum",
				"range": ('tbf','sfq','htb'),
				"description": "Type of egress",
				"name": "egress",
			},
			"$2": {
				"type": "string",
				"description": "Egress ID (optional)",
				"name": "id",
				"mask": pmask.Optional,
			}
		}
		modules = [
			"rate",
			"limit",
			"burst",
			"perturb",
			"quantum",
			["class","category"],
			["class-factory","fcategory"],
			"default",
		]
		acquire = [
			"enable",
		]
		export = [
			"rate",
			"burst",
			"ceil",
		]

		def pre(self,opts):
			command.pre(self,opts)
			opts.envl.fabric = utils.opts({"table": 1,"clsid": 2})

		def postDown(self,opts):
			command.postDown(self,opts)

			result = swrapper()
			dev = opts["envl"]["interface"]
			result.add_commands(
				"exec", ("tc qdisc del dev %s root" % (dev),)
			)

			return result

		def postUp(self,opts):
			command.postUp(self,opts)

			result = swrapper()
			dev = opts.envl.interface


			if opts.envl.egress == "tbf":
				# default values:
				# speed: bit/s
				rate = self.fetch(opts.rest_fc,"rate","128Kbit")
				# size: bytes
				limit = self.fetch(opts.rest_fc,"limit","10240")
				burst = self.fetch(opts.rest_fc,"burst",limit)

				result.add_commands("exec",
					[
						"tc qdisc del dev %s root"
						% (dev),
						"tc qdisc add dev %s handle 1: root tbf limit %s burst %s rate %s" 
						% (dev,limit,burst,rate),
					]
				)
			elif opts.envl.egress == "sfq":
				# default values:
				limit = self.fetch(opts.rest_fc,"limit",128)
				perturb = self.fetch(opts.rest_fc,"perturb",10)
				quantum = self.fetch(opts.rest_fc,"quantum",1500)
				result.add_commands("exec",
					[
						"tc qdisc del dev %s root"
						% (dev),
						"tc qdisc add dev %s handle 1: root sfq limit %s perturb %s quantum %s"
						% (dev,limit,perturb,quantum),
					]
				)
			elif opts.envl.egress == "htb" and opts.envl.has_key("classes"):
				default = int(self.fetch(opts.rest_fc,"default",1)) * 10
				rate = self.fetch(opts.rest_fc,"rate","1Mbit")
				ceil = self.fetch(opts.rest_fc,"ceil",rate)
				burst = self.fetch(opts.rest_fc,"burst","15K")
				result.add_commands("exec",
					[
						"tc qdisc del dev %s root"
						% (dev),
						"tc qdisc add dev %s root handle 1: htb default %s"
						% (dev,default),
						"tc class add dev %s parent 1: classid 1:1 htb rate %s ceil %s burst %s"
						% (dev,rate,ceil,burst),
					]
				)
				result.add_commands("exec",opts["envl"]["classes"])

				if opts.envl.has_key("filters"):
					result.add_commands("exec",opts["envl"]["filters"])


			return result


		class child (command):
			"""
			Common procedures for TBF children

			This class is not to be used directly. It is just a base class
			for conditional egress children.
			"""
			variants = []

			def condition(self,condition):
				# FIXME: one MUST override condition() in these classes
				# (htb class and class-factory) instead of such a hack
				if condition.has_key("category") or condition.has_key("net"):
					return True

				if condition.has_key("egress"):
					if condition["egress"] in self.variants:
						return True

				return False
			
			def gen_class(self,opts,clsid):
				"""
				Return a string for tc class creation
				
				That's needed only for category and fcategory classes
				"""
				dev = opts.envl.interface
				rate = self.fetch(opts.rest_fc,"rate","64Kbit")
				ceil = self.fetch(opts.rest_fc,"ceil",rate)
				burst = self.fetch(opts.rest_fc,"burst",1500)

				if not opts.envl.has_key("classes"):
					opts.envl.classes = []
				if not opts.envl.has_key("filters"):
					opts.envl.filters = []

				opts.envl.classes.append(
					"tc class add dev %s parent 1:1 classid 1:%s htb rate %s ceil %s burst %s"
					% (dev,clsid,rate,ceil,burst),
				)
				opts.envl.classes.append(
					"tc qdisc add dev %s handle %s:0 parent 1:%s sfq perturb 10"
					% (dev,clsid,clsid),
				)

		class fcategory (child):
			"""
			HTB traffic control class factory
			"""
			name = "class-factory"
			variants = ["htb"]
			acquire = [
				"rate",
				"ceil",
				"burst",
			]
			parameters = {
				"$1": {
					"type": "ipaddr",
					"description": "IP network",
					"name": "net",
				},
			}
			mask = flags.Begin | flags.LocalsTransparent
			
			def postUp(self,opts):
				l = ip.ip_range(opts.envl.net)
				mask = ip.get_mask(opts.envl.net)
				mask = ip.msk[32] & ~mask
				dev = opts.envl.interface
				divisor = len(l)

				prio = 10

				if not opts.envl.has_key("filters"):
					opts.envl.filters = []

				opts.envl.fabric.table += 1
				table = opts.envl.fabric.table
				clsid = opts.envl.fabric.clsid

				opts.envl.filters.append(
					"tc filter add dev %s parent 1:0 prio 5 handle %s: protocol ip u32 divisor %s" % 
					(dev,table,divisor)
				)
				opts.envl.filters.append(
					"tc filter add dev %s protocol ip parent 1:0 prio 5 u32 ht 800:: match ip dst %s hashkey mask 0x%08x at 16 link %s:" %
					(dev, opts.envl.net, mask, table)
				)


				for (i,k) in l:
					self.gen_class(opts,clsid)
					opts.envl.filters.append(
						"tc filter add dev %s protocol ip parent 1:0 prio %s u32 ht %s:%s: match ip dst %s flowid 1:%s" %
						(dev,prio,table,i,k,clsid)
					)
					clsid += 1

				opts.envl.fabric.clsid = clsid
				

		class category (child):
			"""
			HTB traffic control class
			"""
			name = "class"
			variants = ["htb"]
			acquire = [
				"rate",
				"ceil",
				"burst",
			]
			modules = [
				"match",
				"priority",
			]
			mask = flags.Begin | flags.LocalsTransparent

			parameters = {
				"$1": {
					"type": "int",
					"description": "Class number",
					"name": "category"
				}
			}

			def postUp(self,opts):
				dev = opts["envl"]["interface"]
				clsid = int(opts["envl"]["category"]) * 10
				self.gen_class(opts,clsid)

				prio = self.fetch(opts.rest_fc,"priority",10)

				if opts["envl"].has_key("matches"):
					for i in opts["envl"]["matches"]:

						# work on match filters (U32)
						ext = ""

						if i.has_key("src"):
							ext += " match ip src %s" % (i["src"])

						if i.has_key("dst"):
							ext += " match ip dst %s" % (i["dst"])

						if i.has_key("sport"):
							ext += " match ip sport %s 0xffff" % (i["sport"])

						if i.has_key("dport"):
							ext += " match ip dport %s 0xffff" % (i["dport"])

						if i.has_key("proto"):
							ext += " match ip protocol %s 0xff" % (i["proto"])

						if ext:
							opts.envl.filters.append(
								"tc filter add dev %s parent 1:0 protocol ip prio %s u32 %s flowid 1:%s"
								% (dev,prio,ext,clsid)
							)

						# filters (fwmark)
						if i.has_key("fwmark"):
							opts.envl.filters.append(
								"tc filter add dev %s protocol ip parent 1:0 prio %s handle %s fw flowid 1:%s"
								% (dev,prio,i["fwmark"],clsid)
							)

						# filters (L7)
						if i.has_key("l7"):
							# create or obtain l7 label
							if not opts.envg.has_key("l7"):
								opts.envg["l7"] = {}
								opts.envg["l7base"] = []

							if not opts.envg.l7.has_key(dev):
								opts.envg.l7[dev] = []
								opts.envg.l7base.append(dev)

							# create tc hooks
							for k in i["l7"]:
								if not k in opts.envg.l7[dev]:
									opts.envg.l7[dev].append(k)
								mark = opts.envg.l7[dev].index(k) + (opts.envg.l7base.index(dev) + 1) * 1024

								opts.envl.filters.append(
									"tc filter add dev %s protocol ip parent 1:0 prio %s handle %s fw flowid 1:%s"
									% (dev,prio,mark,clsid)
								)

							# raise a signal
							self.signal("l7",dev)

			class priority (command):
				"""
				Filter's priority for this category
				"""
				mask = flags.Unique
				parameters = {
					"$1": {
						"type": "int",
						"description": "Priority number",
						"name": "priority",
					}
				}

			class match (classifier):
				"""
				Match pattern
				"""
				parameters = {
					"l7": {
						"description": "L7 filter",
						"type": "word",
						"name": "l7",
						"mask": pmask.Optional | pmask.List,
					},
				}

				def __init__(self):
					command.__init__(self)
					utils.merge(classifier.parameters,self.parameters)

			        def pre(self,opts):
					command.pre(self,opts)
					if not opts["envl"].has_key("matches"):
						opts["envl"]["matches"] = []
					opts["envl"]["matches"].append(opts["opts"].parameters)

		class default (child):
			"""
			Default class
			"""
			mask = flags.Unique | flags.Upload
			variants = ["htb"]
			parameters = {
				"$1": {
					"type": "int",
					"description": "Default class number",
					"name": "default",
				}
			}

		class rate (child):
			"""
			Rate of egress
			"""
			mask = flags.Unique | flags.Upload
			variants = ["tbf","htb"]
			parameters = {
				"$1": {
					"type": "speed",
					"description": "Rate of the egress",
					"name": "rate",
				}
			}

		class ceil (child):
			"""
			Ceil rate of egress
			"""
			mask = flags.Unique | flags.Upload
			variants = ["htb"]
			parameters = {
				"$1": {
					"type": "speed",
					"description": "Ceil rate of the HTB egress",
					"name": "ceil",
				}
			}

		class limit (child):
			"""
			Number of bytes that can be queued waiting for tokens to become available
			"""
			mask = flags.Unique | flags.Upload
			variants = ["tbf","sfq"]
			parameters = {
				"$1": {
					"type": "size",
					"description": "Size of the queue in bytes (tbf) or packets (sfq)",
					"name": "limit",
				}
			}

		class burst (child):
			"""
			Size of the bucket
			"""
			mask = flags.Unique | flags.Upload
			variants = ["tbf","htb"]
			parameters = {
				"$1": {
					"type": "size",
					"description": "Size of the bucket, in bytes",
					"name": "burst",
				}
			}

		class perturb (child):
			"""
			Interval of queue reconfiguration
			"""
			mask = flags.Unique | flags.Upload
			variants = ["sfq"]
			parameters = {
				"$1": {
					"type": "int",
					"description": "Interval, in seconds",
					"name": "perturb",
				}
			}

		class quantum (child):
			"""
			Amount of bytes that can be dequeued in one turn
			"""
			maks = flags.Unique | flags.Upload
			variants = ["sfq"]
			parameters = {
				"$1": {
					"type": "size",
					"description": "Amount, in bytes",
					"name": "quantum",
				}
			}
