# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from command import command, swrapper
from cxutil import flags,utils,cmask,pmask
from cxutil.exceptions import Dump
from cxutil.UID import UID
try:
	###
	# that's a workaround for rpm buildreq bug :|
	###
	from interfaces.template import template
except:
	raise

class Default (command):
	"""
	Configure network interfaces
	"""
	mask = flags.Begin
	name = "interfaces"
	fbase = "interfaces"
	export = [
		"address",
		"hwaddr",
		"shwaddr",
		"speed",
		"use",
		"vlan",
	]

	def postUp(self,opts):
		###
		#
		utils.merge(opts.rest_fc,opts.rest_tp)
		#
		###

## 8<---------------------------------------------------------------------------------

	class address (command):
		"""
		Add an IP address to interface
		"""
		parameters = {
			"$1": {
				"type": "ipaddr",
				"description": "IP address to add",
				"name": "address",
				"dict": "key",
			},
			
			"$2": {
				"type": "enum",
				"description": "address option",
				"name": "address",
				"range": ("propagate",),
				"dict": "value",
				"mask": pmask.Optional
			}
		}

	class shwaddr (command):
		"""
		Set hardware address
		"""
		mask = flags.Unique
		parameters = {
			"$1": {
				"type": "word",
				"description": "Hadware address",
				"name": "shwaddr",
			}
		}

	class hwaddr (command):
		"""
		Match hardware address (to rename)
		"""
		mask = flags.Unique
		parameters = {
			"$1": {
				"type": "word",
				"description": "Hadware address",
				"name": "hwaddr",
			}
		}

	class speed (command):
		"""
		Force speed (may fail on some NICs)
		"""
		mask = flags.Unique
		parameters = {
			"$1": {
				"type": "enum",
				"range": ('10','100','1000'),
				"description": "Force speed on the interface",
				"name": "speed"
			}
		}

	class use (command):
		"""
		Use option (see completion) on the interface
		"""
		parameters = {
			"$1": {
				"type": "enum",
				"range": ('dhcp',),
				"description": "Available options for the interface",
				"name": "option",
				"mask": pmask.List,
			}
		}

	class vlan(template):
		"""
		Create a VLAN on interface
		"""

		parameters = {
			"$1": {
				"description": "VLAN number",
				"type": "int",
				"name": "vlan",
			}
		}
		acquire = [
			"address",
			"egress",
		]

		def pre(self,opts):
			result = template.pre(self,opts)
			if not opts.call_mask & cmask.Immediate:
				opts.envl.subdev = opts.envl.interface
				opts.envl.interface = "%s.%s" % (opts.envl.interface, opts.envl.vlan)
			return result

		def init(self,opts):
			return template.init(self,opts)
