# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from command import swrapper
from command import command
from template import template,normalize
from cxutil import flags,pmask
import re

class bridge (template):
	"""
	Configure a bridge
	"""
	priority = 0x590
	parameters = {
		"$1": {
			"type": "word",
			"description": "bridge number or name",
			"name": "interface",
		}
	}

	iprefix = "br"
	
	acquire = [
		"address",
		"egress",
		"enable",
		"use",
		"vlan",
	]

	modules = [
		"interface",
	]



	def init(self,opts):
		result = template.init(self,opts)
		if opts["envl"].has_key("subdev"):
			dev = opts["envl"]["subdev"]
		else:
			dev = opts["envl"]["interface"]
		result.insert_commands("exec",["brctl addbr %s" % (dev),])
		return result

	def postUp(self,opts):
		result = template.postUp(self,opts)
		if not result:
			result = swrapper()

		if opts.rest_fc.has_key("subint"):
			for i in opts.rest_fc.subint:
				self.add_signal_filter("dev",i)
				result.insert_commands("exec",[
					"brctl addif %s %s" % (opts["envl"]["interface"],i),
				])
		return result

	class interface (template):
		'''
		Add an interface to the bridge
		'''
		catch = []
		parameters = {
			"$1": {
				"type": "string",
				"name": "interface",
				"description": "An interface name"
			},
			"hwaddr": {
				"type": "word",
				"description": "Hardware address",
				"mask": pmask.Optional,
			},
			"set-hwaddr": {
				"type": "word",
				"description": "Hardware address",
				"mask": pmask.Optional,
				"name": "shwaddr",
			},
		}
		mask = flags.StopLocals

		def pre(self,opts):
			result = template.pre(self,opts)
			opts.envl.interface = normalize(opts.envl.interface)
			if not opts.rest_tp.has_key("subint"):
				opts.rest_tp.subint = []
			opts.rest_tp.subint.append(opts.envl.interface)
			return result

