# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from command import command,swrapper
from cxutil import utils,flags,pmask
from cxutil.exceptions import Dump

from ppp import ppp
from template import template,normalize

class pppoe (ppp):
	"""
	Configure PPPoE connection
	"""
	parameters = {
		"$1": {
			"type": "word",
			"description": "Connection name",
			"name": "pppn",
			"mask": pmask.Fname
		}
	}
	
	modules = [
		"service",
		"acn",
		"user",
		("clamp-mss","clamp_mss"),
		"option",
		"interface",
	]


	def postUp(self,opts):
		self.merge_options(opts)

		opts.envl.option["name"] = opts.envl.user

		name = opts.envl.pppn
		optstring = self.format_options(opts)
		interface = normalize(opts.envl.interface)
		poe = " "

		if opts.envl.has_key("service"):
			poe += "-S %s " % (opts.envl.service)
		if opts.envl.has_key("acn"):
			poe += "-C %s " % (opts.envl.acn)
		if opts.envl.has_key("clamp_mss"):
			poe += "-m %s " % (opts.envl.clamp_mss)

		result = swrapper()
		# clear ppp line
		result.merge((self.clean(opts),))
		# ensure pppd is down and restart it again
		result.add_command("system",
			'pppd linkname %s %s pty "pppoe -I %s %s"' % (name,optstring,interface,poe),
		)
		return result

	# 8<--------------------------------------------------------------------------------
	class service (command):
		"""
		Service to discover
		"""
		parameters = {"$1": {"type": "word", "description": "Service name", "name": "service"}}

	class acn (command):
		"""
		Access concentrator name
		"""
		parameters = {"$1": {"type": "word", "description": "ACN", "name": "acn"}}

	class clamp_mss (command):
		"""
		Clamp MSS to this value
		"""
		parameters = {"$1": {"type": "int", "description": "Size in bytes, e.g. 1280", "name": "clamp_mss"}}

	class interface (template):
		"""
		Interface to use
		"""
		mask = 0
		parameters = {
			"$1": { "type": "string", "name": "interface", "description": "An interface name"},
			"hwaddr": { "type": "word", "description": "Hardware address", "mask": pmask.Optional}
		}

	class user (command):
		"""
		User name to use in pap/chap secret
		"""
		parameters = {"$1": {"type": "string", "description": "Host IP or name", "name": "user"}}

	class option (command):
		"""
		User name to use in pap/chap secret
		"""
		parameters = {
			"$1": {"type": "word", "description": "PPP daemon option name", "name": "option", "dict": "key"},
			"$2": {"type": "word", "description": "PPP daemon option value", "name": "option", "dict": "value", "default": True},
		}
