# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from command import command,swrapper
from cxutil import utils,flags,pmask
from cxutil.exceptions import Dump

from ppp import ppp

class pptp (ppp):
	"""
	Configure PPTP tunnel connection
	"""
	parameters = {
		"$1": {
			"type": "word",
			"description": "Tunnel name",
			"name": "pppn",
			"mask": pmask.Fname
		}
	}
	
	modules = [
		"host",
		"user",
		"option",
	]


	# 8<--------------------------------------------------------------------------------
	def postUp(self,opts):
		self.merge_options(opts)

		opts.envl.option["name"] = opts.envl.user

		optstring = self.format_options(opts)
		name = opts.envl.pppn
		host = opts.envl.host

		result = swrapper()
		# clear ppp line
		result.merge((self.clean(opts),))
		# ensure pppd is down and restart it again
		result.add_command("system",
			'pppd linkname %s %s pty "pptp %s --nolaunchpppd"' % (name,optstring,host),
		)
		return result

	# 8<--------------------------------------------------------------------------------
	class host (command):
		"""
		Host to connect to
		"""
		parameters = {"$1": {"type": "word", "description": "Host IP or name", "name": "host"}}

	class user (command):
		"""
		User name to use in pap/chap secret
		"""
		parameters = {"$1": {"type": "string", "description": "Host IP or name", "name": "user"}}

	class option (command):
		"""
		User name to use in pap/chap secret
		"""
		parameters = {
			"$1": {"type": "word", "description": "PPP daemon option name", "name": "option", "dict": "key"},
			"$2": {"type": "word", "description": "PPP daemon option value", "name": "option", "dict": "value", "default": True},
		}
