# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from command import command
from command import swrapper
from cxutil import flags,utils,cmask
from cxutil.exceptions import BranchPass
import re
import os

no_import = True

class template (command):

	mask = flags.Begin
	iprefix = ""
	catch = ["netlink.link.add"]


	def pre(self,opts):
		result = command.pre(self,opts)
		opts.envl.rename = False
		if opts.envl.has_key("interface"):
			if re.match("^[0-9.]+$",opts.envl.interface):
				opts.envl.interface = self.iprefix + opts.envl.interface
			else:
				opts.envl.rename = True

		return result

	def vlan_detect(self,opts):
		dev = opts.envl.interface
		if re.match("^[a-z]+[0-9]+\.[0-9]+",dev):
			opts.envl.vlan = re.sub("^.+\.","",dev)
			opts.envl.subdev = re.sub("\..+$","",dev)

	def vlan_add(self,opts):
		result = swrapper()
		result.add_commands("exec",
			[
				"modprobe 8021q",
				"ip link set dev %s up" % (opts.envl.subdev),
				"vconfig set_name_type DEV_PLUS_VID_NO_PAD",
				"vconfig add %s %s" % (opts.envl.subdev, opts.envl.vlan),
			]
		)
		return result

	def vlan_remove(self,opts):
		result = swrapper()
		result.add_commands("exec",
			[
				"vconfig rem %s" % (opts.envl.interface),
			]
		)
		return result

	def init(self,opts):
		result = swrapper()
		self.vlan_detect(opts)
		if opts.envl.has_key("vlan"):
			result.merge((self.vlan_add(opts),))

		return result

	def postUp(self,opts):
		'''
		BIG FAT WARNING:
			this procedure is self-inducting!
			
			any interface change causes rtnetlink signal that triggers
			this postUp(), that MUST NOT change interface without
			necessity
		'''
		command.postUp(self,opts)

		###
		#
		utils.merge(opts.rest_fc,opts.rest_tp)
		#
		###

		result = swrapper()


		###
		# Currently, only hwaddr-based renaming are supported
		###
		if opts.envl.has_key("hwaddr"):
			r = swrapper()

			if not opts.shared.has_key("iftab"):
				opts.shared.iftab = {}


				# FIXME:
				#	* add internal rtnetlink event listener
				#	* add to cxnet utility calls like "get links"
				#	* replace Executor with rtnetlink call
				#	* ...

				x = utils.Executor("ip -o link show")
				###
				# Here all interface names are. We have:
				#  * hwaddr of the interface
				#  * current name of the interface
				#  * desirable name of the interface
				#
				# We must:
				#  * find, if there is such interface and rename it
				#  * find the interface by hwaddr and give the name to it
				###
				for i in x.lines:
					dev = i.split(":")[1].strip()
					oldname = dev
					###
					# FIXME: tries to work for all interfaces, even w/o link/ether
					###
					hwaddr = i[i.find("link/ether") + 11:i.find("link/ether") + 28]

					opts.shared.iftab[oldname] = utils.opts({
						"name": dev,
						"hwaddr": hwaddr,
					})

					if dev == opts.envl.interface:
						if hwaddr == opts.envl.hwaddr:
							continue
						
						newname = hwaddr.replace(":","")
						r.insert_commands("exec",[
							"ip link set dev %s down" % (dev),
							"ifrename -i %s -n %s" % (dev,newname),
						])
						opts.shared.iftab[oldname]["name"] = dev

					if  hwaddr == opts.envl.hwaddr:
						r.add_commands("exec",[
							"ip link set dev %s down" % (dev),
							"ifrename -i %s -n %s" % (dev, opts.envl.interface),
						])
						opts.shared.iftab[oldname]["name"] = opts.envl.interface

			else:
				print "current: %s" % (opts.envl.interface)
				for (i,k) in opts.shared.iftab.items():
					print "%s: %s" % (i,k)
					if k.name == opts.envl.interface:
						if k.hwaddr == opts.envl.hwaddr:
							continue

						newname = k.hwaddr.replace(":","")
						r.insert_commands("exec",[
							"ip link set dev %s down" % (k.name),
							"ifrename -i %s -n %s" % (k.name,newname),
						])
						k.name = newname
					if k.hwaddr == opts.envl.hwaddr:
						r.add_commands("exec",[
							"ip link set dev %s down" % (k.name),
							"ifrename -i %s -n %s" % (k.name, opts.envl.interface),
						])
						k.name = opts.envl.interface
						break
				
			result.merge((r,))

		dev = opts.envl.interface


		if opts.envl.has_key("shwaddr"):
			r = swrapper()
			r.add_commands("exec",[
				"ip link set dev %s down" % (dev),
				"ip link set dev %s address %s" % (dev,opts.envl.shwaddr)
			])
			result.merge((r,))


		###
		#
		# subscribe for hardware address or for device name
		# note, that it is XOR: we do not use device name when renaming device
		# why? just because we can get a race in flip renaming (e.g. eth0 <-> eth1)
		#
		if opts.envl.has_key("hwaddr"):
			# netlink merges zeroes, so, we'll do it too for filtering
			self.add_signal_filter("hwaddr",re.sub('00','0',opts.envl.hwaddr))
		else:
			self.add_signal_filter("dev",dev)


		if opts["envl"].has_key("address"):
			for (i,k) in opts["envl"]["address"].items():
				result.add_command("exec","ip address add dev %s %s" % (dev,i))
				if k == "propagate":
					try:
						i = i[:i.index("/")]
					except:
						pass
					result.add_command("exec","arping -b -A -s %s -c 5 -I %s 0.0.0.0" % (i,dev))

		if opts["envl"].has_key("speed"):
			result.add_commands("exec",
				[
					"ethtool -s %s speed %s" % (dev,opts["envl"]["speed"]),
				]
			)

		result.add_commands("exec",
			[
				"ip link set dev %s up" % (dev),
			]
		)

		if opts["envl"].has_key("option"):
			for i in opts["envl"]["option"]:
				if i == 'dhcp':
					## choose, which dhcp-client we must use
					for k in os.getenv("PATH").split(":"):

						if os.access("%s/dhcpcd" % (k), os.X_OK):
							opts["persist"]["dhcp client"] = "dhcpcd"
							l = "dhcpcd "
							if opts.envg.has_key("kernel.hostname"):
								l += "-h %s " % (opts.envg["kernel.hostname"])
							l += "%s" % (opts.envl.interface)
							result.add_command("exec",l)
							break

						elif os.access("%s/udhcpc" % (k), os.X_OK):
							opts["persist"]["dhcp client"] = "udhcpc"
							result.add_command("exec","udhcpc -i %s -b -p /var/run/udhcpc.%s.pid" % (opts["envl"]["interface"],opts["envl"]["interface"]))
							break

						elif os.access("%s/dhclient" % (k), os.X_OK):
							opts["persist"]["dhcp client"] = "dhclient"
							if opts.envg.has_key("kernel.hostname"):
								result.add_commands("system",(
									"sed -n -i -e '/interface \"%s\" *{[^}]*}/d;/interface \"%s\" *{[^}]*$/ {:l;n;s/^[^}]*$//;tl;/^[^}]*}/d;};p' /etc/dhcp/dhclient.conf" % (opts.envl.interface,opts.envl.interface),
									"echo 'interface \"%s\" { send fqdn.fqdn \"%s.\"; send fqdn.encoded on; send fqdn.server-update off; }' >> /etc/dhcp/dhclient.conf" % (opts.envl.interface, opts.envg["kernel.hostname"]),
								))
							result.add_command("exec","dhclient -nw -cf /etc/dhcp/dhclient.conf")
							break

		return result
	
	def postDown(self,opts):
		command.postDown(self,opts)
		dev = opts["envl"]["interface"]
		result = swrapper()
		if opts["envl"].has_key("address"):
			for (i,k) in opts["envl"]["address"].items():
				result.add_commands("exec",
					[
						"ip address del dev %s %s" % (dev,i),
					]
				)
		if opts["envl"].has_key("option"):
			for i in opts["envl"]["option"]:
				if i == 'dhcp':
					## choose, which dhcp-client we must use
					if opts["persist"]["dhcp client"] == "dhcpcd":
						del opts["persist"]["dhcp client"]
						result.add_commands("system",("kill `cat /var/run/dhcpcd-%s.pid`" % (opts["envl"]["interface"]),))

					elif opts["persist"]["dhcp client"] == "udhcpc":
						del opts["persist"]["dhcp client"]
						result.add_commands("system",("kill `cat /var/run/udhcpc.%s.pid`" % (opts["envl"]["interface"]),))

					elif opts["persist"]["dhcp client"] == "dhclient":
						del opts["persist"]["dhcp client"]
						result.add_command("system","kill `cat /var/run/dhclient.pid`")

		return result


	def terminate(self,opts):
		self.vlan_detect(opts)

		result = swrapper()
		result.add_commands("exec",
			(
				"ip addr flush dev %s" % (opts.envl.interface),
				"ip link set dev %s down" % (opts.envl.interface),
			)
		)
		if opts.envl.has_key("vlan"):
			result.merge((self.vlan_remove(opts),))
		return result




def normalize(name):
	if re.match("^ethernet.*",name):
		num = re.sub("^[^0-9]+","",name)
		return "eth" + num
	return name
