# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from template import template
from command import command,not_ready,swrapper
from cxutil import flags

class wireless (template):
	"""
	Configure Wi-Fi interface
	"""
	priority = 0x500
	catch = []

	parameters = {
		"$1": {
			"type": "word",
			"description": "interface number or name",
			"name": "interface",
		}
	}

	modules = [
		"base",
		"driver",
		"mode",
		"channel",
		"essid",
		"ap",
		"eapol",
		"proto",
		("key-management","keymgmt"),
		"psk",
	]

	acquire = [
		"address",
		"hwaddr",
		("set-hwaddr","shwaddr"),
		"egress",
		"enable",
		"use",
		"vlan",
	]


	class driver (command):
		"""
		Force wpa_supplicant driver
		"""
		mask = flags.Unique
		parameters = {
			"$1": {
				"type": "enum",
				"range": ('wext','hostap','prism54','madwifi','atmel','ndiswrapper','ipw','wired'),
				"description": "Available drivers",
				"name": "driver"
			}
		}

	class mode (not_ready):
		"""
		Interface mode
		"""
		mask = flags.Unique
		parameters = {
			"$1": {
				"type": "enum",
				"range": ('ad-hoc','managed','master','repeater','secondary','monitor','auto'),
				"description": "Avalable modes",
				"name": "mode"
			}
		}

	class channel (command):
		"""
		Force channel to use
		"""
		pass

	class essid (command):
		"""
		Choose ESSID
		"""
		pass

	class ap (command):
		"""
		Force access point address
		"""
		pass

	class eapol (not_ready):
		"""
		Force EAPOL version (default = 1)
		"""
		pass

	class proto (not_ready):
		"""
		Accepted protocols list
		"""
		pass

	class keymgmt (not_ready):
		"""
		Force key management
		"""
		pass

	class psk (command):
		"""
		Pre-shared key
		"""
		pass

