# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from command import command,swrapper
from cxutil import flags,pmask
from cxutil.exceptions import Dump

class table (command):
	"""
	Define routing table
	"""
	mask = flags.Begin

	parameters = {
		"$1": {
			"type": "word",
			"description": "Routing table",
			"default": "main",
			"mask": pmask.Optional,
			"name": "table"
		},
		"$2": {
			"type": "word",
			"description": "Call ID",
			"mask": pmask.Optional | pmask.Fname,
			"name": "cid",
		}
	}
	
	modules = [
		"route",
	]
	
	subscribe = [
		"xshell configure network interfaces ethernet",
		"xshell configure network interfaces bridge",
		"xshell configure network interfaces loopback",
	]

	def post(self,opts,method):

		result = swrapper()

		f = "/etc/iproute2/rt_tables"
		if opts["envl"].has_key("table"):
			table = opts["envl"]["table"]
		else:
			table = "main"

		system = ["local","main","default","unspec"]


		if method == "replace":
			free = range(256)
			tables = {}

			## check for table
			try:
				rt = open(f,"r")
			except Exception,e:
				result.add_commands("message",[e,])
				return result

			for i in rt.readlines():
				if i[0] == "#":
					continue
				i = i.split()
				if len(i) != 2:
					continue
				if int(i[0]) in free:
					free.remove(int(i[0]))
					tables[i[1]] = int(i[0])

			rt.close()

			## if no such table, create an ID for it
			if not table in tables:
				result.add_commands("pedit",
					[
						"file %s" % (f),
						"replace %s" % (free[-1:][0]),
						"%s" % (table)
					]
				)


		if opts["envl"].has_key("route"):
			for i in opts["envl"]["route"]:
				result.add_commands("exec",("ip route %s table %s %s" % (method,table,i),))
		
		if (method == "del") and (not table in system):
			result.add_commands("system",("cat %s | grep -v '\t%s$' >%s.new; mv -f %s.new %s" % (f,table,f,f,f),))

		result.add_commands("exec",("ip route flush cache",))

		return result

	def postUp(self,opts):
		command.postUp(self,opts)
		return self.post(opts,"replace")

	def postDown(self,opts):
		command.postDown(self,opts)
		return self.post(opts,"del")


##	subclasses

	class route (command):
		"""
		Define route
		"""
	
		parameters = {
			"$1": {
				"type": "string",
				"description": "Route description (see ip(8) from iproute2)",
				"name": "route",
				"mask": pmask.List,
			}
		}

